#!/usr/bin/tclsh

set version 1.0
set version 1.1
set version 2.0
set version 2.1  ;# extract booth
set version 2.2  ;# mark deleted games
set version 3.0  ;# extract bgglink again; add language column
set version 3.1  ;# remove empty lines with &nbsp;
set version 3.2  ;# corrected lang and prio columns
set version 3.3  ;# print spielbox link in first line
set version 4.0  ;# extract spielen.de link
set version 5.0  ;# column for price
set version 5.1  ;# do not mark games with remark "BGG" as deleted
set version 5.2  ;# read bgg website to

# Initialaufruf:
# ./update_messe.tcl webseite.html > datei.csv
# Danach:
# ./update_messe.tcl webseite.html bgg.html datei.csv > neue_datei.csv

if { $argc < 1 } {
    puts "Error: Keine HTML-Datei angegeben!"
    exit 1
}

set webpage    [lindex $argv 0]
set webpageBGG ""
set csvfile    ""

# puts "Args: $argc $argv"

if { $argc > 1 } {
    if { [regexp {.html$} [lindex $argv 1]] } {
        set webpageBGG [lindex $argv 1]
        if { $argc > 2 } {
            if { [regexp {.csv$} [lindex $argv 2]] } {
                set csvfile [lindex $argv 2]
            } elseif { [string length [lindex $argv 2]] > 0 } {
                puts "Unknown file: [lindex $argv 2]"
                exit 1
            }
        }
    } elseif { [regexp {.csv$} [lindex $argv 1]] } {
        set csvfile [lindex $argv 1]
    } else {
        puts "Unknown file: [lindex $argv 1]"
        exit 1
    }
}

# -----------------------------------------------

proc readCsvEntry { line } {
    global g_csvVersion
    global g_newList
    global g_changedList
    global g_boothList
    global g_publisherList
    global g_gameList
    global g_playersList
    global g_ageList
    global g_durationList
    global g_languageList
    global g_priceList
    global g_prioList
    global g_remarkList
    global g_timestampList
    global g_bggLinkList
    global g_spielenLinkList
    global g_everythingChangedList
    global g_inCsvOnly

    if { $g_csvVersion >= 1.0 && $g_csvVersion < 2.0 } {
        # v1.0 - v1.1
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Prio,Remarks,LC,BGG"
        if { [regexp {^([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*)$} $line all new changed booth publisher game players age duration prio remarks timestamp bgglink ] } {
            set entry [createEntry $publisher $game]
            set g_newList($entry)       $new
            set g_changedList($entry)   $changed
            set g_boothList($entry)     $booth
            set g_publisherList($entry) $publisher
            set g_gameList($entry)      $game
            set g_playersList($entry)   $players
            set g_ageList($entry)       $age
            set g_durationList($entry)  $duration
            set g_languageList($entry)  {}
            set g_priceList($entry)     {}
            set g_prioList($entry)      $prio
            set g_remarkList($entry)    $remarks
            set g_timestampList($entry) $timestamp
            set g_bggLinkList($entry)   $bgglink
            set g_spielenLinkList($entry) {}
            set g_everythingChangedList($entry) {}
            set g_inCsvOnly($entry)     1
            if { [string compare $g_newList($entry) "X"] == 0 } {
                set g_newList($entry) "Y"
            }
        } else {
            error "Wrong CSV line: $line"
        }
    } elseif { $g_csvVersion >= 2.0 && $g_csvVersion < 3.0 } {
        # v2.0
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Prio,Remarks,Created,BGG"
        if { [regexp {^([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*)$} $line all new changed booth publisher game players age duration prio remarks timestamp ] } {
            set entry [createEntry $publisher $game]
            set g_newList($entry)       $new
            set g_changedList($entry)   $changed
            set g_boothList($entry)     $booth
            set g_publisherList($entry) $publisher
            set g_gameList($entry)      $game
            set g_playersList($entry)   $players
            set g_ageList($entry)       $age
            set g_durationList($entry)  $duration
            set g_languageList($entry)  {}
            set g_priceList($entry)     {}
            set g_prioList($entry)      $prio
            set g_remarkList($entry)    $remarks
            set g_timestampList($entry) $timestamp
            set g_bggLinkList($entry)   {}
            set g_spielenLinkList($entry) {}
            set g_everythingChangedList($entry) {}
            set g_inCsvOnly($entry)     1
            if { [string compare $g_newList($entry) "X"] == 0 } {
                set g_newList($entry) "Y"
            }
        } else {
            error "Wrong CSV line: $line"
        }
    } elseif { $g_csvVersion >= 3.0 && $g_csvVersion < 4.0 } {
        # v3.0
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Prio,Remarks,Created,BGG"
        if { [regexp {^([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*)$} $line all new changed booth publisher game players age duration language prio remarks timestamp bgglink ] } {
            set entry [createEntry $publisher $game]
            set g_newList($entry)       $new
            set g_changedList($entry)   $changed
            set g_boothList($entry)     $booth
            set g_publisherList($entry) $publisher
            set g_gameList($entry)      $game
            set g_playersList($entry)   $players
            set g_ageList($entry)       $age
            set g_durationList($entry)  $duration
            set g_languageList($entry)  $language
            set g_priceList($entry)     {}
            set g_prioList($entry)      $prio
            set g_remarkList($entry)    $remarks
            set g_timestampList($entry) $timestamp
            set g_bggLinkList($entry)   $bgglink
            set g_spielenLinkList($entry) {}
            set g_everythingChangedList($entry) {}
            set g_inCsvOnly($entry)     1
            if { [string compare $g_newList($entry) "X"] == 0 } {
                set g_newList($entry) "Y"
            }
        } else {
            error "Wrong CSV line: $line"
        }
    } elseif { $g_csvVersion >= 4.0 && $g_csvVersion < 5.0 } {
        # v4.0
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Prio,Remarks,Created,BGG,spielen.de"
        if { [regexp {^([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*)$} $line all new changed booth publisher game players age duration language prio remarks timestamp bgglink spielenlink ] } {
            set entry [createEntry $publisher $game]
            set g_newList($entry)       $new
            set g_changedList($entry)   $changed
            set g_boothList($entry)     $booth
            set g_publisherList($entry) $publisher
            set g_gameList($entry)      $game
            set g_playersList($entry)   $players
            set g_ageList($entry)       $age
            set g_durationList($entry)  $duration
            set g_languageList($entry)  $language
            set g_priceList($entry)     {}
            set g_prioList($entry)      $prio
            set g_remarkList($entry)    $remarks
            set g_timestampList($entry) $timestamp
            set g_bggLinkList($entry)   $bgglink
            set g_spielenLinkList($entry) $spielenlink
            set g_everythingChangedList($entry) {}
            set g_inCsvOnly($entry)     1
            if { [string compare $g_newList($entry) "X"] == 0 } {
                set g_newList($entry) "Y"
            }
        } else {
            error "Wrong CSV line: $line"
        }
    } elseif { $g_csvVersion >= 5.0 && $g_csvVersion < 6.0 } {
        # v5.0
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Price,Prio,Remarks,Created,BGG,spielen.de"
        if { [regexp {^([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*)$} $line all new changed booth publisher game players age duration language price prio remarks timestamp bgglink spielenlink ] } {
            set entry [createEntry $publisher $game]
            set g_newList($entry)       $new
            set g_changedList($entry)   $changed
            set g_boothList($entry)     $booth
            set g_publisherList($entry) $publisher
            set g_gameList($entry)      $game
            set g_playersList($entry)   $players
            set g_ageList($entry)       $age
            set g_durationList($entry)  $duration
            set g_languageList($entry)  $language
            set g_priceList($entry)     $price
            set g_prioList($entry)      $prio
            set g_remarkList($entry)    $remarks
            set g_timestampList($entry) $timestamp
            set g_bggLinkList($entry)   $bgglink
            set g_spielenLinkList($entry) $spielenlink
            set g_everythingChangedList($entry) {}
            set g_inCsvOnly($entry)     1
            if { [string compare $g_newList($entry) "X"] == 0 } {
                set g_newList($entry) "Y"
            }
        } else {
            error "Wrong CSV line: $line"
        }
    } else {
        error "readCsvEntry Unknown version: $g_csvVersion"
    }
}

proc checkCsvVersion { file } {
    global g_csvVersion
    gets $file line
    regexp {^v([^,]*),} $line all g_csvVersion
    #puts $g_csvVersion
    gets $file line
    if { $g_csvVersion >= 1.0 && $g_csvVersion < 2.0 } {
        # v1.x
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Prio,Remarks,LC,BGG"
        if { [string compare $line "N,C,Booth,Publisher,Game,Players,Age,Dur,Prio,Remarks,LC,BGG"] != 0 } {
            error "CSV-format does not match version $g_csvVersion: $line"
        }
    } elseif { $g_csvVersion >= 2.0 && $g_csvVersion < 3.0 } {
        # v2.x
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Prio,Remarks,Created"
        if { [string compare $line "N,C,Booth,Publisher,Game,Players,Age,Dur,Prio,Remarks,Created"] != 0 } {
            error "CSV-format does not match version $g_csvVersion: $line"
        }
    } elseif { $g_csvVersion >= 3.0 && $g_csvVersion < 4.0 } {
        # v3.x
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Prio,Remarks,Created,BGG"
        if { [string compare $line "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Prio,Remarks,Created,BGG"] != 0 } {
            error "CSV-format does not match version $g_csvVersion: $line"
        }
    } elseif { $g_csvVersion >= 4.0 && $g_csvVersion < 5.0 } {
        # v4.x
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Prio,Remarks,Created,BGG,spielen.de"
        if { [string compare $line "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Prio,Remarks,Created,BGG,spielen.de"] != 0 } {
            error "CSV-format does not match version $g_csvVersion: $line"
        }
    } elseif { $g_csvVersion >= 5.0 && $g_csvVersion < 6.0 } {
        # v5.x
        # puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Price,Prio,Remarks,Created,BGG,spielen.de"
        if { [string compare $line "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Price,Prio,Remarks,Created,BGG,spielen.de"] != 0 } {
            error "CSV-format does not match version $g_csvVersion: $line"
        }
    } else {
        error "checkCsvVersion Unknown version: $g_csvVersion"
    }
}

proc readCsv { filename } {
    global g_csvVersion
    set file [open $filename r]
    checkCsvVersion $file
    while { [gets $file line] >= 0 } {
        readCsvEntry $line
    }
}

# -----------------------------------------------

proc convertUnicode { line } {
    regsub -all {,} $line {} line
    regsub -all {&amp;} $line {and} line
    regsub -all { & } $line { and } line
    regsub -all {&#039;} $line {} line
    regsub -all {\?\?\?} $line {Fragezeichen} line
    regsub -all {\?} $line {} line
    regsub -all {–} $line {-} line
    regsub {^[ \t]*} $line {} line
    regsub {[ \t]*$} $line {} line
    regsub {               } $line {} line
    return $line
}

proc createEntry { publisher game } {
    set entry "$publisher$game"
    regsub -all {,} $entry {} entry
    regsub -all {\.} $entry {} entry
    regsub -all {!} $entry {} entry
    regsub -all {\?} $entry {} entry
    regsub -all {:} $entry {} entry
    regsub -all {'} $entry {} entry
    regsub -all {\-} $entry {} entry
    regsub -all { } $entry {} entry
    regsub -all {\(} $entry {} entry
    regsub -all {\)} $entry {} entry
    regsub -all {/} $entry {} entry
    regsub -all {­­­} $entry {} entry
    regsub -all {\+} $entry {} entry
    regsub -all {\+} $entry {} entry
    regsub {^[0-9]*} $entry {} entry
    return $entry
}

proc checkChanged { entry oldValue newValue hasChanged } {
    global g_everythingChangedList
    
    upvar $oldValue localOldValue
    upvar $newValue localNewValue
    upvar $hasChanged localHasChanged
    
    if { [info exists localNewValue] } {
        if { [string compare $localOldValue $localNewValue] } {
            if { [string length $localOldValue] > 0 } {
                set localHasChanged 1
            }
            lappend g_everythingChangedList($entry) "<$localOldValue> -> <$localNewValue>"
            set localOldValue $localNewValue
        }
        unset localNewValue
    }
}

proc storeGame { } {

    global g_newList
    global g_changedList
    global g_boothList
    global g_publisherList
    global g_gameList
    global g_playersList
    global g_ageList
    global g_durationList
    global g_languageList
    global g_priceList
    global g_prioList
    global g_remarkList
    global g_timestampList
    global g_bggLinkList
    global g_spielenLinkList
    global g_everythingChangedList
    global g_inCsvOnly

    global g_publisher
    global g_booth
    global g_game
    global g_players
    global g_playtime
    global g_age
    global g_bgglink
    global g_spielenlink
    global g_timestamp

    global g_counter

    if { [info exists g_publisher] && [info exists g_game] } {
        set entry [createEntry $g_publisher $g_game]
        set isNew 0
        incr g_counter
        #puts "$g_counter $entry"
        if { ![info exists g_gameList($entry)] } {
            # Spiel existiert noch nicht in der Liste
            set isNew 1
            set g_newList($entry)        {}
            set g_changedList($entry)    {}
            set g_boothList($entry)      {}
            set g_publisherList($entry)  {}
            set g_gameList($entry)       {}
            set g_playersList($entry)    {}
            set g_durationList($entry)   {}
            set g_ageList($entry)        {}
            set g_languageList($entry)   {}
            set g_priceList($entry)      {}
            set g_prioList($entry)       {}
            set g_remarkList($entry)     {}
            set g_timestampList($entry)  {}
            set g_bggLinkList($entry)    {}
            set g_spielenLinkList($entry) {}
            set g_everythingChangedList($entry) {}
            set g_inCsvOnly($entry)      0
        } else {
            set g_inCsvOnly($entry)      0
        }

        set g_publisherList($entry) $g_publisher
        # Publisher nicht loeschen, da der nicht wiederholt wird!

        if { [info exists g_booth] } {
            if { [string compare $g_boothList($entry) $g_booth] } {
                #if { ![string compare $g_boothList($entry) ""] } {
                #    set hasChanged 1
                #}
                set g_boothList($entry) $g_booth
            }
            # Booth nicht loeschen, da es nicht wiederholt wird
        }

        set g_gameList($entry) $g_game
        unset g_game

        set hasChanged 0
        checkChanged $entry g_playersList($entry) g_players hasChanged
        checkChanged $entry g_durationList($entry) g_playtime hasChanged
        checkChanged $entry g_ageList($entry) g_age hasChanged
        checkChanged $entry g_bggLinkList($entry) g_bgglink hasChanged
        checkChanged $entry g_spielenLinkList($entry) g_spielenlink hasChanged
        # checkChanged $entry g_timestampList($entry) g_timestamp hasChanged
        set g_timestampList($entry) $g_timestamp
        
        if { $isNew } {
            set g_newList($entry) "X"
        }
        # else lassen wir den vorherigen Eintrag stehen
        if { $hasChanged } {
            set g_changedList($entry) "X"
        }
    }
}

proc checkPublisher { line } {
    global g_publisher
    global g_booth
    global g_game
    global g_publisherStarted
    if { $g_publisherStarted } {
        #puts "$line"
        if { [regexp {<a.*class="publisherlink"[^>]*>([^<]*)</a>} $line all publisher] } {
            set publisher [convertUnicode $publisher] 
            #puts "> $publisher"
            set g_publisher $publisher
        } else {
            regsub -all {<[^>]*>} $line {} line
            set publisher [convertUnicode $line]
            if { [string length $publisher] > 0 } {
                #puts ">> $publisher"
                set g_publisher $publisher
            }
        }
        if { [info exists g_booth] } {
            unset g_booth
        }
    }
}

proc checkGameTitle { line } {
    global g_game
    global g_gameStarted
    global g_gameNameStarted
    if { $g_gameStarted && $g_gameNameStarted } {
        # puts "$line"
        regsub -all {<[^>]*>} $line {} line
        set game [convertUnicode $line]
        if { [string length $game] > 0 } {
            set g_game $game
        }
    }
}

proc checkPlayers { line } {
    global g_players
    global g_gameStarted
    if { $g_gameStarted } {
        regexp {, (.*) Spieler} $line all g_players
    }
}

proc checkPlaytime { line } {
    global g_playtime
    global g_gameStarted
    if { $g_gameStarted } {
        regexp {, Spieldauer ca. (.*) Minuten} $line all g_playtime
    }
}

proc checkAge { line } {
    global g_age
    global g_gameStarted
    if { $g_gameStarted } {
        regexp {ab (.*) Jahren} $line all g_age
    }
}

proc checkBooth { line } {
    global g_booth
    if { [regexp {\(Messestand: *([^\)]*) *\)} $line all g_booth] } {
        set g_booth [convertUnicode $g_booth]
        regsub {^0([0-9])-([A-Z])-(.*)$} $g_booth {\1-\2\3} g_booth
        # puts "|$g_booth|"
    }
}

proc checkBggLink { line } {
    global g_bgglink
    if { [regexp {<a href="([^"]*)"[^>]*>BGG</a>} $line all g_bgglink] } {  ;#"
        # puts $g_bgglink
    }
}

proc checkTimestamp { line } {
    global g_timestamp
    global g_gameStarted
    if { $g_gameStarted } {
        if { [regexp {<nobr>\[([^\]]*)\]</nobr>} $line all g_timestamp] } {
            # puts $g_timestamp
        }
    }
}

proc checkSpielenLink { line } {
    global g_spielenlink
    if { [regexp {<a href="([^"]*)"[^>]* class="info2">mehr Informationen[^<]*</a>} $line all g_spielenlink] } {  ;#"
        # puts $g_bgglink
    }
}

proc checkMode { line } {
    global g_publisherStarted
    global g_gameListStarted
    global g_gameStarted
    global g_gameNameStarted
    
    if { [regexp {<h2>} $line] } {
        if { $g_publisherStarted } {
            error "Zweiten Publisher über <h2> gefunden!"
        }
        set g_publisherStarted 1
    }
    if { [regexp {</h2>} $line] } {
        if { !$g_publisherStarted } {
            error "Publisher beendet über </h2> ohne Start!"
        }
        set g_publisherStarted 0
    }
    if { [regexp {<ul>} $line] } {
        if { $g_gameListStarted } {
            error "Zweite Spieleliste begonnen über <ul> gefunden!"
        }
        set g_gameListStarted 1
    }
    if { [regexp {</ul>} $line] } {
        if { !$g_gameListStarted } {
            error "Spieleliste beendet über </ul> ohne Start!"
        }
        set g_gameListStarted 0
    }
    if { [regexp {<li>} $line] } {
        if { !$g_gameListStarted } {
            error "Spieleliste hat noch nicht begonnen!"
        }
        if { $g_gameStarted } {
            error "Zweites Spiel begonnen über <li> gefunden!"
        }
        set g_gameStarted 1
    }
    if { [regexp {</li>} $line] } {
        if { !$g_gameListStarted } {
            error "Spieleliste hat noch nicht begonnen!"
        }
        if { !$g_gameStarted } {
            error "Spiel beendet über </li> ohne Start!"
        }
        set g_gameStarted 0
        storeGame
    }
    if { [regexp {<strong>} $line] } {
        if { !$g_gameListStarted } {
            error "Spieleliste hat noch nicht begonnen!"
        }
        if { !$g_gameStarted } {
            error "Spiel hat noch nicht begonnen!"
        }
        if { $g_gameNameStarted } {
            error "Zweiter Spielname begonnen über <strong> gefunden!"
        }
        set g_gameNameStarted 1
    }
    if { [regexp {</strong>} $line] } {
        if { !$g_gameListStarted } {
            error "Spieleliste hat noch nicht begonnen!"
        }
        if { !$g_gameStarted } {
            error "Spiel hat noch nicht begonnen!"
        }
        if { !$g_gameNameStarted } {
            error "Spielname beendet über </strong> ohne Start!"
        }
        set g_gameNameStarted 0
    }
}

proc createList { line } {
    # Wir erstellen aus der Zeile eine Liste mit den einzelnen Datensegmenten.
    set sep µ
    if { [regexp $sep $line] } {
        error "Zeile $line enthält Trennzeichen $sep."
    }
    regsub -all {<a>} $line "$sep<a>" line
    regsub -all {</a>} $line "</a>$sep" line
    regsub -all {<h2>} $line "$sep<h2>$sep" line
    regsub -all {</h2>} $line "$sep</h2>$sep" line
    regsub -all {<ul>} $line "$sep<ul>$sep" line
    regsub -all {</ul>} $line "$sep</ul>$sep" line
    regsub -all {<li>} $line "$sep<li>$sep" line
    regsub -all {</li>} $line "$sep</li>$sep" line
    regsub -all {<strong>} $line "$sep<strong>$sep" line
    regsub -all {</strong>} $line "$sep</strong>$sep" line
    regsub -all {<nobr>} $line "$sep<nobr>" line
    regsub -all {</nobr>} $line "</nobr>$sep" line
    regsub -all {&nbsp;} $line " " line
    regsub -all {^\s*$} $line "" line
    return [split $line $sep]
}

proc readHtmlSpielbox { filename } {
    global g_publisherStarted
    global g_gameListStarted
    global g_gameStarted
    global g_gameNameStarted
    global g_counter

    set g_publisherStarted 0
    set g_gameListStarted 0
    set g_gameStarted 0
    set g_gameNameStarted 0
    set g_counter 0
    
    set file [open $filename r]
    while { [gets $file line_from_file] >= 0 } {
        set list_of_lines [createList $line_from_file] 
        foreach line $list_of_lines {
            checkMode $line
            checkPublisher $line
            checkBooth $line
            checkGameTitle $line
            checkPlayers $line
            checkPlaytime $line
            checkAge $line
            checkBggLink $line
            checkTimestamp $line
            checkSpielenLink $line
        }
    }
}

proc convertToSpielboxFormat { line } {
    
    # Publisher
    regsub {かぼへる \(Kaboheru\)} $line {Kaboheru} line
    regsub {新ボードゲーム党 \(New Board Game Party\)} $line {New Board Game Party} line
    regsub {AMIGO Spiel \+ Freizeit GmbH} $line {Amigo Spiel + Freizeit GmbH} line
    regsub {Arclight} $line {Arclight Games} line
    regsub {ADC Blackfire Entertainment GmbH} $line {ADC Blackfire} line
    regsub {Asmodee} $line {Asmodee Deutschland} line
    regsub {Backspindle Games Ltd.} $line {Backspindle Games} line
    regsub {Bezier Games Inc.} $line {Bézier Games} line
    regsub {Blue Cocker Games} $line {Blue Cocker} line
    regsub {Cool Mini Or Not} $line {Cool Mini or Not} line
    regsub {franjos Spieleverlag} $line {Franjos Spieleverlag} line
    regsub {Hans im Glück Verlags-GmbH} $line {Hans im Glück Verlag} line
    regsub {HUCH! and friends} $line {Huch! and friends} line
    regsub {IELLO} $line {Iello} line
    regsub {KOSMOS} $line {Kosmos} line
    regsub {Lautapelit.fi} $line {Lautapelit} line
    regsub {PLAYthisONE} $line {Playthisone} line
    regsub {Schmidt Spiele} $line {Schmidt Spiele GmbH} line
    regsub {Board&Dice} $line {Board and Dice} line
    regsub {GIFT10INDUSTRY} $line {Gift10Industry} line
    regsub {KYY Games} $line {Kyy Games} line
    regsub {Tasty Minstrel Games} $line {Tasty Minstrel Games (TMG)} line

    # Spiele
    regsub {KerFlip!} $line {Kerflip!} line
    regsub {Elfenroads} $line {Elfenland de luxe} line
    regsub {Project: ELITE} $line {Project: Elite} line
    regsub {Tank Hunter 2e Cmdr} $line {Tank Hunter 2e: Cmdr} line
    regsub {Tank Hunter 2e Jäger} $line {Tank Hunter 2e: Jäger} line
    regsub {Clacks: A Discworld Board Game} $line {Clacks: A Discworld Game} line
    regsub {Glenn Drover's Empires: Age of Discovery - Deluxe Edition} $line {Empires: Age of Discovery} line
    regsub {Wharfside} $line {Fleet: Wharfside} line
    regsub {Zombie Dice} $line {Zombie Würfel Deluxe} line
    regsub {Zombie Dice 3: School Bus} $line {Zombie Würfel 3: Schulbus} line
    regsub {Tides of Time} $line {Tides of Time: Im Strom der Zeit} line
    regsub {Star Munchkin 3: Diplomatic Impunity} $line {Star Munchkin 3: Diplomatische Immunität} line
    regsub {Munchkin Treasure Hunt} $line {Munchkin Verborgene Schätze} line
    regsub {Mage Wars: Forged in Fire - Spell Tome Expansion} $line {Mage Wars Arena: Flammenschmiede} line 
    regsub {Lost Legacy: Second Chronicle - Vorpal Sword and Whitegold Spire} $line {Lost Legacy: Todesschwert and Weißgoldturm} line
    regsub {Asterix & Obelix Mau Mau} $line {Asterix and Obelix: Mau Mau} line
    regsub {Suburbia 5★} $line {Suburbia: 5 Sterne} line
    regsub {Consiglio dei Quattro} $line {Council of Four} line
    regsub {La Torre dei Monstri} $line {The Tower} line
    regsub {Through the Ages: A New Story of Civilization} $line {Through the Ages - Zweite Auflage} line
    regsub {Rally Up!} $line {Boost} line
    regsub {Raccoon Rumpus} $line {Klamotten Klamauk} line
    regsub {The Sneaky Snacky Squirrel Game} $line {Fitz Stibitz} line
    regsub {Nyet!} $line {Njet!} line
    regsub {Star Wars: Angriff der Rebellen} $line {Star Wars Rebels: Angriff der Rebellen} line
    regsub {Cube Quest} $line {Rumms} line
    regsub {Tumult Royale} $line {Tumult Royal} line
    regsub {Wits and Wagers} $line {Top and Flop} line
    regsub {Idol Conclave \+ Million hit Chart} $line {Idol Conclave} line
    regsub {動物夜怕怕 \(Animals Frightening Night!\)} $line {Animals Frightening Night!} line
    regsub {RYŪ} $line {Ryu} line
    regsub {Steam Ship Company} $line {Steam Ship Company (Prototyp)} line
    regsub {Mage Wars: Forged in Fire - Spell Tome Expansion} $line {Mage Wars Arena: Flammenschmiede} line
    regsub {Zombie Würfel Deluxe 3: School Bus} $line {Zombie Würfel 3: Schulbus} line
    regsub {Die Holde Isolde} $line {Die holde Isolde} line
    regsub {Ligretto} $line {Ligretto: Star Wars Rebels} line
    regsub {Qwirkle} $line {Qwirkle: Star Wars} line
    regsub {Team Play} $line {Teamplay} line
    regsub {T.I.M.E Stories} $line {Time Stories} line
    regsub {The Golden Ages} $line {Goldene Zeitalter} line
    regsub {A Study in Emerald \(second edition\)} $line {Eine Studie in Smaragdgrün} line
    regsub {8-28} $line {8*28} line

    return $line
}

proc checkPublisherBGG { line } {
   global g_publisher
   if { [regexp {Board Game Publisher: <a.[^>]*>([^<]*)</a>} $line all publisher] } {
        set publisher [convertUnicode $publisher]
        set publisher [convertToSpielboxFormat $publisher]
        # puts "> $publisher"
        set g_publisher $publisher
    }
}

proc checkGameTitleBGG { line } {
    global g_game
    if { [regexp {Board Game[^:]*: <a [^>]*>([^<]*)</a>} $line all game] } {
        set game [convertUnicode $game]
        set game [convertToSpielboxFormat $game]
        # puts "$game"
        if { [string length $game] > 0 } {
            set g_game $game
        }
    }
}

proc checkBggLinkBGG { line } {
    global g_bgglink
    if { [regexp {Board Game[^:]*: <a [^h]*href="([^"]*)".*</a>} $line all link] } {
        set g_bgglink "https://boardgamegeek.com$link"
        # puts "$g_bgglink"
    }
}
#" ; Kommentar wegen Syntax-Highlighting-Fehler in Geany

proc checkTimestampBGG { line } {
    global g_timestamp
    if { [regexp {added by.*on ([^<]*)</span>} $line all g_timestamp] } {
        # TODO: Konvertiere "2015-08-10 11:56:07" in "10.08.2015 11:56"
        # puts "$g_timestamp"
        return 1
    }
    return 0
}

proc readHtmlBGG { filename } {
    global g_counter
    set g_counter 0
    set file [open $filename r]
    while { [gets $file line_from_file] >= 0 } {
        checkPublisherBGG $line_from_file
        checkGameTitleBGG $line_from_file
        checkBggLinkBGG   $line_from_file
        if { [checkTimestampBGG $line_from_file] } {
            storeGame
        }
    }
}

# -----------------------------------------------

proc printEntries { } {

    global g_newList
    global g_changedList
    global g_boothList
    global g_publisherList
    global g_gameList
    global g_playersList
    global g_ageList
    global g_durationList
    global g_languageList
    global g_priceList
    global g_prioList
    global g_remarkList
    global g_timestampList
    global g_bggLinkList
    global g_spielenLinkList
    global g_everythingChangedList
    global g_inCsvOnly
    global version

    puts "v$version,,,,,,,,," ;#,,,Extracted from:,http://partner.spielen.de/spielbox/messeneuheiten/verlag/2015/essen,,,,,,,,,
    puts "N,C,Booth,Publisher,Game,Players,Age,Dur,Lang,Price,Prio,Remarks,Created,BGG,spielen.de"

    foreach entry [lsort -nocase -ascii [array names g_newList]] {
        puts -nonewline $g_newList($entry)
        puts -nonewline ","
        if { !$g_inCsvOnly($entry) } {
            puts -nonewline $g_changedList($entry)
        } else {
            if { [string first "BGG" $g_remarkList($entry)] == -1 } { 
                puts -nonewline "D"
            }
        }
        puts -nonewline ","
        puts -nonewline $g_boothList($entry)
        puts -nonewline ","
        puts -nonewline $g_publisherList($entry)
        puts -nonewline ","
        puts -nonewline $g_gameList($entry)
        puts -nonewline ","
        puts -nonewline $g_playersList($entry)
        puts -nonewline ","
        puts -nonewline $g_ageList($entry)
        puts -nonewline ","
        puts -nonewline $g_durationList($entry)
        puts -nonewline ","
        puts -nonewline $g_languageList($entry)
        puts -nonewline ","
        puts -nonewline $g_priceList($entry)
        puts -nonewline ","
        puts -nonewline $g_prioList($entry)
        puts -nonewline ","
        puts -nonewline $g_remarkList($entry)
        puts -nonewline ","
        puts -nonewline $g_timestampList($entry)
        puts -nonewline ","
        puts -nonewline $g_bggLinkList($entry)
        puts -nonewline ","
        puts -nonewline $g_spielenLinkList($entry)
        #puts -nonewline " "
        #puts -nonewline $g_everythingChangedList($entry)
        puts ""
    }
}

# -----------------------------------------------

if { [string length $csvfile] > 0 } {
    readCsv $csvfile
}

readHtmlSpielbox $webpage
readHtmlBGG $webpageBGG

printEntries





